"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function emitStatefull(info) {
    var source = ((info.hasAct && info.hasState)
        ? "import { \n    ActionFn, \n    ReducibleComponent, \n    buildComponent, \n    ActionResult, \n    Self, \n    PerformEffect \n} from 'ts-react-api';"
        : "import { \n            StatelessComponent\n} from 'ts-react-api';") + "\nimport { " + (info.hasAct ? 'Action, ' : '') + (info.hasState ? 'State, ' : '') + "make } from './" + info.componentName + "';\n\nexport type Props = {\n";
    if (info.makeParameters) {
        info.makeParameters.forEach(function (par) {
            source += "    " + par.name.getText() + ": " + (par.type ? par.type.getText() : 'any') + ";\n";
        });
    }
    source += "}\n\n" + ((info.hasAct && info.hasState) ? "export interface Component extends ReducibleComponent<Action, Props, State> { }\n\nexport interface ComponentEffect {\n    noUpdate(): ActionResult;\n    update(state: State): ActionResult;\n    sideEffects(fn: (self: Self<Action, Props, State>) => void): ActionResult;\n    updateWithSideEffects(state: State, fn: (self: Self<Action, Props, State>) => void): ActionResult;\n};\n\nexport const Effect = PerformEffect as ComponentEffect;" : 'export interface Component extends StatelessComponent { }') + "\n\n";
    if (info.hasAct && info.hasState && info.actFuncs) {
        info.actFuncs.forEach(function (fn) {
            var pars = fn.parameters;
            source += "export function " + fn.name.getText() + "(" + pars.map(function (p) { return p.name.getText() + ": " + p.type.getText(); }).join(', ') + "): ActionFn<Action> {\n    return {\n        match: function (act: Action): ActionResult {\n            return act." + fn.name.getText() + "(" + pars.map(function (p) { return "" + p.name.getText(); }).join(', ') + ");\n        }\n    }\n}\n\n";
        });
    }
    if (info.hasAct && info.hasState) {
        source += "export const " + info.componentName + " = buildComponent<Props, Action, State>(props => make(" + (info.makeParameters
            ? info.makeParameters.map(function (p) { return "props." + p.name.getText(); }).join(', ')
            : '') + "));\n";
    }
    else {
        source += "export const " + info.componentName + " = (props: Props) => make(" + (info.makeParameters
            ? info.makeParameters.map(function (p) { return "props." + p.name.getText(); }).join(', ')
            : '') + ").render();";
    }
    return source;
}
exports.emitStatefull = emitStatefull;
//# sourceMappingURL=emiter.js.map