"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var path = require("path");
var fs_1 = require("fs");
function extractInfo(fileName) {
    var sourceFile = ts.createSourceFile(fileName, fs_1.readFileSync(fileName).toString(), ts.ScriptTarget.ES2015, true);
    var makeParameters;
    var actFuncs;
    var hasAct = false;
    var hasState = false;
    function visit(sourceFile) {
        traverse(sourceFile);
        function traverse(node) {
            switch (node.kind) {
                case ts.SyntaxKind.VariableDeclaration:
                    var varDeclr = node;
                    if (varDeclr.name.getText() == 'make' && varDeclr.initializer.kind == ts.SyntaxKind.ArrowFunction) {
                        var fnType = varDeclr.initializer;
                        makeParameters = fnType.parameters;
                    }
                    return;
                case ts.SyntaxKind.TypeAliasDeclaration:
                    var aliasDeclr = node;
                    var typ = aliasDeclr.type;
                    if (aliasDeclr.name.getText() == 'Action' && typ.kind == ts.SyntaxKind.TypeLiteral) {
                        hasAct = true;
                        var litTyp = typ;
                        actFuncs = litTyp.members.filter(function (m) { return m.kind == ts.SyntaxKind.MethodSignature; });
                    }
                    else if (aliasDeclr.name.getText() == 'State') {
                        hasState = true;
                    }
                    return;
            }
            ts.forEachChild(node, traverse);
        }
    }
    visit(sourceFile);
    var componentName = path.parse(fileName).name;
    return {
        componentName: componentName,
        makeParameters: makeParameters,
        actFuncs: actFuncs,
        hasAct: hasAct,
        hasState: hasState
    };
}
exports.extractInfo = extractInfo;
//# sourceMappingURL=info.js.map