#!/usr/bin/env node
import * as fs from 'fs';
import * as path from 'path';
import { getTsReactFiles } from './util';
import { emitStatefull } from './emiter';
import { extractInfo } from './info';

const meow = require('meow');
const watchr = require('watchr');
const firstline = require('firstline');

type Cli = {
    flags: {
        watch: boolean;
        w: boolean;
        help: boolean;
        h: boolean;
    },
    input: string[];
    showHelp: () => void;
}

const cli: Cli = meow(`
	Usage
	  $ tsr [path]

	Options
      --watch, -w  watch the path for changes
      --help, -h  show this help
`, {
        flags: {
            watch: {
                type: 'boolean',
                alias: 'w'
            }
        }
    });

async function emitComponent(fileName: string) {
    if (!(/^(\s*\/\/\s*@ts\-react\s*)$/.test(await firstline(fileName)))) {
        return;
    }

    const info = extractInfo(fileName);

    const outName = `${path.parse(fileName).dir}/${path.parse(fileName).name}.gen.tsx`;
    console.info(` - ${path.parse(fileName).name} > ${outName}`);
    fs.writeFileSync(outName, emitStatefull(info));
}

async function listener(changeType: string, fullPath: string, currentStat: string, previousStat: string) {
    if (changeType == 'update') {
        if (fullPath.indexOf('node_modules') === -1 && !fullPath.endsWith('.gen.tsx') && fullPath.endsWith('.tsx')) {
            await emitComponent(fullPath);
        }
    }
}

function next(err: any) {
    if (err) return console.error('watch failed on', err)
}

if (process.argv.length == 3 && (process.argv[2] == '-h' || process.argv[2] == '--help')) {
    cli.showHelp();
} else {
    let startPath = null;

    if (cli.input.length > 0) {
        startPath = cli.input[0];
    } else {
        startPath = process.cwd();
        console.info(`Using: ${startPath}`);
    }

    const stats = fs.lstatSync(startPath);
    if (!stats.isDirectory()) {
        console.info(`Invalid directory: ${startPath}`);
    } else {
        console.info('Looking for files with "// @ts-react" annotation...');
        getTsReactFiles(startPath).forEach(async fileName => {
            await emitComponent(fileName);
        });

        if (cli.flags.watch || cli.flags.w) {
            console.info('Watch mode started...');
            const stalker = watchr.open(startPath, listener, next);
            process.on('SIGTERM', () => {
                console.info('Stopping...');
                stalker.close();
            });
        }
    }
}