import { Info } from './info';

export function emitStatefull(info: Info) {
    let source = `${(info.hasAct && info.hasState)
        ? `import { 
    ActionFn, 
    ReducibleComponent, 
    buildComponent, 
    ActionResult, 
    Self, 
    PerformEffect 
} from 'ts-react-api';`
        : `import { 
            StatelessComponent
} from 'ts-react-api';`}
import { ${info.hasAct ? 'Action, ' : ''}${info.hasState ? 'State, ' : ''}make } from './${info.componentName}';

export type Props = {
`;

    if (info.makeParameters) {
        info.makeParameters.forEach(par => {
            source += `    ${par.name.getText()}: ${par.type ? par.type.getText() : 'any'};\n`;
        });
    }

    source += `}

${((info.hasAct && info.hasState) ? `export interface Component extends ReducibleComponent<Action, Props, State> { }

export interface ComponentEffect {
    noUpdate(): ActionResult;
    update(state: State): ActionResult;
    sideEffects(fn: (self: Self<Action, Props, State>) => void): ActionResult;
    updateWithSideEffects(state: State, fn: (self: Self<Action, Props, State>) => void): ActionResult;
};

export const Effect = PerformEffect as ComponentEffect;` : 'export interface Component extends StatelessComponent { }')}

`;

    if (info.hasAct && info.hasState && info.actFuncs) {
        info.actFuncs.forEach(fn => {
            const pars = fn.parameters;

            source += `export function ${fn.name.getText()}(${pars.map(p => `${p.name.getText()}: ${p.type.getText()}`).join(', ')}): ActionFn<Action> {
    return {
        match: function (act: Action): ActionResult {
            return act.${fn.name.getText()}(${pars.map(p => `${p.name.getText()}`).join(', ')});
        }
    }
}

`;
        });
    }

    if (info.hasAct && info.hasState) {
        source += `export const ${info.componentName} = buildComponent<Props, Action, State>(props => make(${
            (info.makeParameters
                ? info.makeParameters.map(p => `props.${p.name.getText()}`).join(', ')
                : '')
            }));
`;    
    } else {
        source += `export const ${info.componentName} = (props: Props) => make(${
            (info.makeParameters
                ? info.makeParameters.map(p => `props.${p.name.getText()}`).join(', ')
                : '')
            }).render();`;
    }

    return source;
}
